@@ -1,7 +1,7 @@
 //
 // BAGEL - Brilliantly Advanced General Electronic Structure Library
 // Filename: dimer_scf.cc
-// Copyright (C) 2014 Shane Parker
+// Copyright (C) 2014 Toru Shiozaki
 //
 // Author: Shane Parker <shane.parker@u.northwestern.edu>
 // Maintainer: NU theory
@@ -313,7 +313,7 @@ void Dimer::set_active(const shared_ptr<const PTree> idata, const bool localize_
   }
 
   nvirt_ = {nexternA - nactA, nexternB - nactB};
-  sref_ = make_shared<Reference>(sgeom_, make_shared<Coeff>(*out_coeff), nclosed, nact, nexternA+nexternB - (nclosed+nact));
+  sref_ = make_shared<Reference>(sgeom_, make_shared<Coeff>(*out_coeff), nclosed, nact, nexternA+nexternB - nact);
 }
 
 
