@@ -103,7 +103,8 @@ class ZHarrison : public Method {
     void save(Archive& ar, const unsigned int) const {
       ar << boost::serialization::base_object<Method>(*this);
       ar << max_iter_ << davidson_subspace_ << thresh_ << print_thresh_ << nele_ << ncore_ << norb_ << charge_
-         << nstate_ << states_ << energy_ << cc_ << space_ << int_space_ << denom_ << rdm1_ << rdm2_ << rdm1_av_ << rdm2_av_ << davidson_ << restart_ << restarted_;
+         << nstate_ << states_ << energy_ << cc_ << space_ << int_space_ << denom_ << rdm1_ << rdm2_ << rdm1_av_ << rdm2_av_ << davidson_
+         << store_half_ints_ << store_gaunt_half_ints_ << restart_ << restarted_;
       // for jop_
       std::shared_ptr<const ZCoeff_Block> coeff = jop_->coeff();
       ar << coeff;
@@ -112,7 +113,8 @@ class ZHarrison : public Method {
     void load(Archive& ar, const unsigned int) {
       ar >> boost::serialization::base_object<Method>(*this);
       ar >> max_iter_ >> davidson_subspace_ >> thresh_ >> print_thresh_ >> nele_ >> ncore_ >> norb_ >> charge_
-         >> nstate_ >> states_ >> energy_ >> cc_ >> space_ >> int_space_ >> denom_ >> rdm1_ >> rdm2_ >> rdm1_av_ >> rdm2_av_ >> davidson_ >> restart_ >> restarted_;
+         >> nstate_ >> states_ >> energy_ >> cc_ >> space_ >> int_space_ >> denom_ >> rdm1_ >> rdm2_ >> rdm1_av_ >> rdm2_av_ >> davidson_
+         >> store_half_ints_ >> store_gaunt_half_ints_ >> restart_ >> restarted_;
       std::shared_ptr<const ZCoeff_Block> coeff;
       ar >> coeff;
       update(coeff);
