@@ -146,8 +146,12 @@ void CASSecond::compute() {
   muffle_->unmute();
 
   // block diagonalize coeff_ in nclosed and nvirt
-  if (max_iter_ > 0)
-    coeff_ = semi_canonical_orb();
+  if (max_iter_ > 0) {
+    auto tmp = semi_canonical_orb();
+    coeff_ = get<0>(tmp);
+    eig_   = get<1>(tmp);
+    occup_ = get<2>(tmp);
+  }
 
   // this is not needed for energy, but for consistency we want to have this...
   // update construct Jop from scratch
@@ -238,9 +242,10 @@ shared_ptr<RotFile> CASSecond::compute_denom(shared_ptr<const DFHalfDist> half,
       Matrix tmp(nao, nao);
       for (int i = 0; i != nclosed_; ++i) {
         dgemv_("T", nri, nao*nao, 1.0, geom_->df()->block(0)->data(), nri, vgcc->block(0)->data()+nri*(i+nclosed_*i), 1, 0.0, tmp.data(), 1);
-        tmp.allreduce();
+        if (!vgcc->serial())
+          tmp.allreduce();
         Matrix tmp0 = vcoeff % tmp * vcoeff;
-        blas::ax_plus_y_n(-4.0, tmp0.diag().get(), nvirt_, denom->ptr_vc()+nvirt_*i);
+        blas::ax_plus_y_n(-4.0, tmp0.diag().data(), nvirt_, denom->ptr_vc()+nvirt_*i);
       }
     }
     shared_ptr<const DFFullDist> vaa  = halfa->compute_second_transform(acoeff);
@@ -250,12 +255,13 @@ shared_ptr<RotFile> CASSecond::compute_denom(shared_ptr<const DFHalfDist> half,
       Matrix tmp(nao, nao);
       for (int i = 0; i != nact_; ++i) {
         dgemv_("T", nri, nao*nao, 1.0, geom_->df()->block(0)->data(), nri, vgaa->block(0)->data()+nri*(i+nact_*i), 1, 0.0, tmp.data(), 1);
-        tmp.allreduce();
+        if (!vgaa->serial())
+          tmp.allreduce();
         Matrix tmp0 = vcoeff % tmp * vcoeff;
-        blas::ax_plus_y_n(2.0, tmp0.diag().get(), nvirt_, denom->ptr_va()+nvirt_*i);
+        blas::ax_plus_y_n(2.0, tmp0.diag().data(), nvirt_, denom->ptr_va()+nvirt_*i);
         if (nclosed_) {
           Matrix tmp1 = ccoeff % tmp * ccoeff;
-          blas::ax_plus_y_n(2.0, tmp1.diag().get(), nclosed_, denom->ptr_ca()+nclosed_*i);
+          blas::ax_plus_y_n(2.0, tmp1.diag().data(), nclosed_, denom->ptr_ca()+nclosed_*i);
         }
       }
       shared_ptr<const DFFullDist> vaa_noj = fci_->jop()->mo2e_1ext()->compute_second_transform(acoeff);
@@ -292,9 +298,10 @@ shared_ptr<RotFile> CASSecond::compute_denom(shared_ptr<const DFHalfDist> half,
       vgaa->ax_plus_y(-1.0, vaa);
       for (int i = 0; i != nact_; ++i) {
         dgemv_("T", nri, nao*nao, 1.0, geom_->df()->block(0)->data(), nri, vgaa->block(0)->data()+nri*(i+nact_*i), 1, 0.0, tmp.data(), 1);
-        tmp.allreduce();
+        if (!vgaa->serial())
+          tmp.allreduce();
         Matrix tmp0 = ccoeff % tmp * ccoeff;
-        blas::ax_plus_y_n(4.0, tmp0.diag().get(), nclosed_, denom->ptr_ca()+nclosed_*i);
+        blas::ax_plus_y_n(4.0, tmp0.diag().data(), nclosed_, denom->ptr_ca()+nclosed_*i);
       }
     }
   }
