@@ -32,6 +32,82 @@ using namespace bagel;
 using namespace bagel::SMITH;
 
 
+void MSCASPT2::MSCASPT2::do_rdm_deriv(double factor) {
+  Timer timer(1);
+  const int nstates = info_->ciwfn()->nstates();
+  std::shared_ptr<Vec<double>> den0cirdm;
+  std::shared_ptr<VecRDM<1>> den1cirdm;
+  std::shared_ptr<VecRDM<2>> den2cirdm;
+  std::shared_ptr<VecRDM<3>> den3cirdm;
+  std::shared_ptr<VecRDM<3>> den4cirdm;
+  tie(den0cirdm, den1cirdm, den2cirdm, den3cirdm, den4cirdm) = feed_denci();
+
+  ci_deriv_ = make_shared<Dvec>(info_->ref()->ciwfn()->det(), nstates);
+  const size_t nact  = info_->nact();
+  const size_t norb2 = nact * nact;
+  const size_t ndet = ci_deriv_->data(0)->size();
+  const size_t ijmax = info_->cimaxchunk();
+  const size_t ijnum = ndet * norb2 * norb2;
+  const size_t npass = ndet < 1000 ? 1 : ((mpi__->size() > ((ijnum-1) / ijmax + 1)) && (mpi__->size() != 1)) ? mpi__->size() : (ijnum - 1) / ijmax + 1;
+  const size_t nsize = (ndet - 1) / npass + 1;
+
+  for (int nst = 0; nst != nstates; ++nst) {
+    if (npass > 1)
+      cout << "       - CI derivative contraction (state " << setw(2) << nst + 1 << ") will be done with " << npass << " passes" << endl;
+
+    // Fock-weighted 2RDM derivative evaluated first (needed for calculating Fock-weighted 3RDM derivative)
+    rdm2fderiv_ = SpinFreeMethod<double>::feed_rdm_2fderiv(info_, fockact_, nst);
+
+    if (npass > 1)
+      timer.tick_print("Fock-weighted 2RDM derivative");
+
+    // embarrasingly parallel mode. npass > 1 -> distribute among the nodes.
+    // otherwise just do using all the nodes.
+    const int nproc = npass > 1 ? 1 : mpi__->size();
+    const int ncomm = mpi__->size() / nproc;
+    const int icomm = mpi__->rank() / nproc;
+    mpi__->split(nproc);
+
+    for (int ipass = 0; ipass != npass; ++ipass) {
+      if (ipass % ncomm == icomm && ncomm != icomm) {
+        const size_t ioffset = ipass * nsize;
+        const size_t isize = (ipass != (npass - 1)) ? nsize : ndet - ioffset;
+        tie(rdm0deriv_, rdm1deriv_, rdm2deriv_, rdm3fderiv_)
+          = SpinFreeMethod<double>::feed_rdm_deriv(info_, fockact_, nst, ioffset, isize, rdm2fderiv_);
+        for (int mst = 0; mst != nstates; ++mst) {
+          den0cirdmt = den0cirdm->at(nst, mst);
+          den1cirdmt = den1cirdm->at(nst, mst);
+          den2cirdmt = den2cirdm->at(nst, mst);
+          den3cirdmt = den3cirdm->at(nst, mst);
+          den4cirdmt = den4cirdm->at(nst, mst);
+          if (info_->shift_imag() && info_->shift() != 0.0) {
+            if (!info_->sssr() || nst == mst) {
+              *(den1cirdmt) += *(etensor1_->at(nst, mst));
+              *(den2cirdmt) += *(etensor2_->at(nst, mst));
+              *(den3cirdmt) += *(etensor3_->at(nst, mst));
+              *(den4cirdmt) += *(etensor4_->at(nst, mst));
+            }
+          }
+
+          shared_ptr<VectorB> bdata = contract_rdm_deriv(info_->ciwfn(), ioffset, isize, fockact_);
+          blas::ax_plus_y_n(factor, bdata->data(), ndet, ci_deriv_->data(mst)->data());
+        }
+
+        if (npass > 1) {
+          stringstream ss; ss << "Multipassing (" << setw(2) << ipass + 1 << " / " << npass << ")";
+          timer.tick_print(ss.str());
+        }
+      }
+    }
+    mpi__->merge();
+  }
+
+  if (npass > 1)
+    for (int mst = 0; mst != nstates; ++mst)
+      mpi__->allreduce(ci_deriv_->data(mst)->data(), ndet);
+}
+
+
 tuple<shared_ptr<Vec<double>>,shared_ptr<VecRDM<1>>,shared_ptr<VecRDM<2>>,shared_ptr<VecRDM<3>>,shared_ptr<VecRDM<3>>> MSCASPT2::MSCASPT2::feed_denci() {
   const int nstates = info_->ciwfn()->nstates();
   const size_t nact  = info_->nact();
