@@ -366,8 +366,10 @@ void MOFock<complex<double>>::init() {
     const int nc = 2;
     assert(f->ndim() == f->mdim() && f->ndim() == nc * info_->nocc() + nc * info_->nvirt());
     f->copy_block(0, 0, nc*info_->nclosed(), nc*info_->nclosed(), fsave->get_submatrix(0, 0, nc*info_->nclosed(), nc*info_->nclosed()));
-    f->copy_block(nc*info_->nclosed(), nc*info_->nclosed(), nc*info_->nact(), nc*info_->nact(), fsave->get_submatrix(nc*info_->nclosed(), nc*info_->nclosed(), nc*info_->nact(), nc*info_->nact()));
-    f->copy_block(nc*info_->nocc(), nc*info_->nocc(), nc*info_->nvirt(), nc*info_->nvirt(), fsave->get_submatrix(nc*info_->nocc(), nc*info_->nocc(), nc*info_->nvirt(), nc*info_->nvirt()));
+    f->copy_block(nc*info_->nclosed(), nc*info_->nclosed(), nc*info_->nact(), nc*info_->nact(),
+                  fsave->get_submatrix(nc*info_->nclosed(), nc*info_->nclosed(), nc*info_->nact(), nc*info_->nact()));
+    f->copy_block(nc*info_->nocc(), nc*info_->nocc(), nc*info_->nvirt(), nc*info_->nvirt(),
+                  fsave->get_submatrix(nc*info_->nocc(), nc*info_->nocc(), nc*info_->nvirt(), nc*info_->nvirt()));
   }
 
   data_ = fill_block<2,complex<double>>(f->get_conjg(), {0,0}, blocks_);
@@ -376,9 +378,9 @@ void MOFock<complex<double>>::init() {
   // print out the Fock matrix and terminate when preparing for external RDM runs
   if (info_->external_rdm() == "noref") {
     stringstream ss; ss << scientific << setprecision(15);
-    for (int i = 0; i != nact; ++i) {
-      for (int j = 0; j != nact; ++j)
-        ss << f->element(j+nclosed+ncore, i+nclosed+ncore) << " ";
+    for (int i = 0; i != nact*2; ++i) {
+      for (int j = 0; j != nact*2; ++j)
+        ss << f->element(j+(nclosed+ncore)*2, i+(nclosed+ncore)*2) << " ";
       ss << endl;
     }
     ofstream fs("FOCKMAT");
@@ -485,10 +487,10 @@ void MOFock<double>::init() {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 // explict instantiation at the end of the file
-template class K2ext<double>;
-template class K2ext<complex<double>>;
-template class MOFock<double>;
-template class MOFock<complex<double>>;
+template class bagel::SMITH::K2ext<double>;
+template class bagel::SMITH::K2ext<complex<double>>;
+template class bagel::SMITH::MOFock<double>;
+template class bagel::SMITH::MOFock<complex<double>>;
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endif
