@@ -55,7 +55,11 @@ class SMITH_Info {
     bool do_xms_;
     bool sssr_;
     bool shift_diag_;
+    bool shift_imag_;
     bool block_diag_fock_;
+    // use orthogonal basis in linear equation for CASPT2 case
+    bool orthogonal_basis_;
+
     bool restart_;
     bool restart_each_iter_;
 
@@ -75,7 +79,7 @@ class SMITH_Info {
     void serialize(Archive& ar, const unsigned int) {
       ar & ref_ & method_ & ncore_ & nfrozenvirt_ & thresh_ & shift_ & maxiter_;
       ar & maxtile_ & cimaxchunk_ & davidson_subspace_ & grad_;
-      ar & do_ms_ & do_xms_ & sssr_ & shift_diag_ & block_diag_fock_ & restart_ & restart_each_iter_;
+      ar & do_ms_ & do_xms_ & sssr_ & shift_diag_ & shift_imag_ & block_diag_fock_ & orthogonal_basis_ & restart_ & restart_each_iter_;
       ar & thresh_overlap_ & state_begin_ & restart_iter_ & aniso_data_ & external_rdm_;
     }
 
@@ -111,9 +115,11 @@ class SMITH_Info {
     bool do_xms() const { return do_xms_; }
     bool sssr() const { return sssr_; }
     bool shift_diag() const { return shift_diag_; }
+    bool shift_imag() const { return shift_imag_; }
     bool block_diag_fock() const { return block_diag_fock_; }
     bool restart() const { return restart_; }
     bool restart_each_iter() const { return restart_each_iter_; }
+    bool orthogonal_basis() const { return orthogonal_basis_; }
     bool rdm4_eval() const { return (grad_ || method_!="caspt2"); }
 
     double thresh_overlap() const { return thresh_overlap_; }
